#! /usr/bin/python

##   backupfiller
##   Copyright (C) 2007 Mads Villadsen <maxximum@krakoa.dk>
   
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
  
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
  
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os, sys

VERSION = "0.2"

MEGABYTE = 1024*1024
GIGABYTE = 1024*MEGABYTE
CD_SIZE = 700*MEGABYTE
DVD_SIZE = 4400*MEGABYTE # 4482MiB according to growisofs man page

class BackupSet:
    def __init__(self, max_size):
        self.__current_size = 0
        self.max_size = max_size # max size in bytes
        self.files = [] # list of {filename, filesize} tuples

    def add_file(self, filename):
        if (os.path.isfile(filename)):
            if (os.path.getsize(filename) + self.__current_size <= self.max_size):
                self.files.append([filename, os.path.getsize(filename)])
                self.__current_size += os.path.getsize(filename)
                return True
        elif (os.path.islink(filename)):
            self.files.append([filename, 0])
            return True
        else:
            return False


def get_sorted_filelist(basedir):
    retval = []
    for root, dirs, files in os.walk(basedir):
        for f in files:
            retval.append(os.path.join(root, f))
    retval.sort()
    return retval

if __name__ == '__main__':
    if (len(sys.argv) == 3):
        # set the desired backup size
        if (sys.argv[1] == "CD"):
            backup_size = CD_SIZE
            # throttle back the size a bit to make sure we end up with iso images that actually fit (95% is simply a guess)           		    backup_size = int(backup_size * 0.95)
        elif (sys.argv[1] == "DVD"):
            backup_size = DVD_SIZE
            # throttle back the size a bit to make sure we end up with iso images that actually fit (95% is simply a guess)
            backup_size = int(backup_size * 0.9)
        else:
            try:
                backup_size = int(sys.argv[1]) * MEGABYTE
            except ValueError:
                print "Not a valid size for the BackupSets: " + sys.argv[1]
                sys.exit(1)

        # set the directory to backup
        dirbase = sys.argv[2]
        
        sets = []
        sets.append(BackupSet(backup_size))

        files = get_sorted_filelist(dirbase)
        for f in files:
            added = sets[-1].add_file(f) # always add to the last set in the list
            if (not added):
                # print "Unable to add: " + f
            
                if (os.path.getsize(f) > backup_size):
                    print "ERROR: File too large to fit in a BackupSet: " + f + " --- this file will not be backed up."
                else:
                    # time to start a new BackupSet
                    # print "Starting new BackupSet"
                    sets.append(BackupSet(backup_size))
                    added = sets[-1].add_file(f)


        print "Generating " + str(len(sets)) + " backup sets"
        for i in range(len(sets)):
            out_f = open("backupset-" + str(i+1) + ".lst", "w")
            for f in sets[i].files:
                reldir = os.path.dirname(f[0])[len(dirbase):]
                fullfile = f[0]

                out_f.write(reldir + "/=" + fullfile + "\n")
            out_f.close()

    else:
        print "backupfiller.py <BackupSet size> <directory to backup>\n"
        print "BackupSet size can be either CD (700MB), DVD (4.7 GB) , or a number specifying how many megabytes each BackupSet should be."
        print ""
        print "backupfiller will create a number of files called backupset-<n>.lst each of which can be used with mkisofs together with the -graft-points -path-list options to create an iso image of the desired size."
        print ""
        print "Usage examples:"
        print "backupfiller.py CD \"/home/maxx/Documents\""
        print "backupfiller.py DVD \"/home/maxx/Documents\""
        print "backupfiller.py 1024 \"/home/maxx/Documents\""
        print ""
        print "Example of using mkisofs:"
        print "mkisofs -joliet-long -r -iso-level 4 -D -graft-points -path-list backupset-1.lst -o backup-1.iso"
        print ""
        print "This is BackupFiller " + VERSION


    # mkisofs -joliet-long -r -iso-level 4 -D -graft-points -path-list foo.lst -o foo.iso
    # growisofs -Z /dev/dvd=foo.iso
